// Flutter imports:
import 'package:flutter/cupertino.dart';

// Project imports:
import 'package:zego_uikit/src/services/internal/core/core.dart';

class ZegoUIKitUser {
  String id = '';
  String name = '';

  Stream<double> get soundLevel {
    if (id == ZegoUIKitCore.shared.coreData.localUser.id) {
      return ZegoUIKitCore.shared.coreData.localUser.soundLevel.stream;
    } else {
      return ZegoUIKitCore.shared.coreData.remoteUsersList
          .firstWhere((user) => user.id == id, orElse: ZegoUIKitCoreUser.empty)
          .soundLevel
          .stream;
    }
  }

  ValueNotifier<Map<String, String>> get inRoomAttributes {
    if (id.isEmpty) {
      return ValueNotifier<Map<String, String>>({});
    }

    if (id == ZegoUIKitCore.shared.coreData.localUser.id) {
      return ZegoUIKitCore.shared.coreData.localUser.inRoomAttributes;
    } else {
      return ZegoUIKitCore.shared.coreData.remoteUsersList
          .firstWhere((user) => user.id == id, orElse: ZegoUIKitCoreUser.empty)
          .inRoomAttributes;
    }
  }

  ZegoUIKitUser.empty();

  bool isEmpty() {
    return id.isEmpty || name.isEmpty;
  }

  ZegoUIKitUser({
    required this.id,
    required this.name,
  });

  // internal helper function
  ZegoUser toZegoUser() => ZegoUser(id, name);

  ZegoUIKitUser.fromZego(ZegoUser zegoUser)
      : this(id: zegoUser.userID, name: zegoUser.userName);

  @override
  String toString() {
    return 'user:{id:$id, name:$name, in-room attributes:${inRoomAttributes.value}';
  }
}
