part of 'zego_uikit_core.dart';

class ZegoUIKitCoreData {
  ZegoUIKitCoreUser localUser = ZegoUIKitCoreUser.localDefault();

  final List<ZegoUIKitCoreUser> remoteUsersList = [];

  final Map<String, ZegoUIKitCoreMixerStream> mixerStreamDic =
      {}; // key:stream_id

  Timer? mixerSEITimer;

  ZegoScreenCaptureSource? screenCaptureSource;

  final Map<String, String> streamDic = {}; // stream_id:user_id

  ZegoUIKitCoreRoom room = ZegoUIKitCoreRoom('');

  bool isAllPlayStreamAudioVideoMuted = false;
  ValueNotifier<bool> isScreenSharing = ValueNotifier(false);

  ZegoAudioVideoResourceMode playResourceMode =
      ZegoAudioVideoResourceMode.Default;

  StreamController<List<ZegoUIKitCoreUser>> userJoinStreamCtrl =
      StreamController<List<ZegoUIKitCoreUser>>.broadcast();
  StreamController<List<ZegoUIKitCoreUser>> userLeaveStreamCtrl =
      StreamController<List<ZegoUIKitCoreUser>>.broadcast();
  StreamController<List<ZegoUIKitCoreUser>> userListStreamCtrl =
      StreamController<List<ZegoUIKitCoreUser>>.broadcast();

  StreamController<List<ZegoUIKitCoreUser>> audioVideoListStreamCtrl =
      StreamController<List<ZegoUIKitCoreUser>>.broadcast();
  StreamController<List<ZegoUIKitCoreUser>> screenSharingListStreamCtrl =
      StreamController<List<ZegoUIKitCoreUser>>.broadcast();

  StreamController<String> meRemovedFromRoomStreamCtrl =
      StreamController<String>.broadcast();
  StreamController<ZegoInRoomCommandReceivedData>
      customCommandReceivedStreamCtrl =
      StreamController<ZegoInRoomCommandReceivedData>.broadcast();
  StreamController<ZegoNetworkMode> networkModeStreamCtrl =
      StreamController<ZegoNetworkMode>.broadcast();

  StreamController<String> turnOnYourCameraRequestStreamCtrl =
      StreamController<String>.broadcast();
  StreamController<String> turnOnYourMicrophoneRequestStreamCtrl =
      StreamController<String>.broadcast();

  StreamController<ZegoUIKitReceiveSEIEvent> receiveSEIStreamCtrl =
      StreamController<ZegoUIKitReceiveSEIEvent>.broadcast();

  ZegoUIKitVideoConfig videoConfig = ZegoUIKitVideoConfig();

  ZegoEffectsBeautyParam beautyParam = ZegoEffectsBeautyParam.defaultParam();

  bool isFirstScreenSharing = true;

  ZegoStreamResourceMode get sdkPlayResourceMode {
    switch (playResourceMode) {
      case ZegoAudioVideoResourceMode.Default:
        return ZegoStreamResourceMode.Default;
      case ZegoAudioVideoResourceMode.CDNOnly:
        return ZegoStreamResourceMode.OnlyCDN;
      case ZegoAudioVideoResourceMode.L3Only:
        return ZegoStreamResourceMode.OnlyL3;
      case ZegoAudioVideoResourceMode.RTCOnly:
        return ZegoStreamResourceMode.OnlyRTC;
      case ZegoAudioVideoResourceMode.CDNPlus:
        return ZegoStreamResourceMode.CDNPlus;
    }
  }

  void clearStream() {
    ZegoLoggerService.logInfo(
      'clear stream',
      tag: 'uikit',
      subTag: 'core data',
    );

    if (isScreenSharing.value) {
      stopSharingScreen();
    }

    for (final user in remoteUsersList) {
      if (user.streamID.isNotEmpty) {
        stopPlayingStream(user.streamID);
      }
      user.destroyTextureRenderer();

      if (user.auxStreamID.isNotEmpty) {
        stopPlayingStream(user.auxStreamID);
      }
      user.destroyTextureRenderer(isMainStream: false);
    }

    if (localUser.streamID.isNotEmpty) {
      stopPublishingStream(streamType: ZegoStreamType.main);
      localUser.destroyTextureRenderer(isMainStream: true);
    }
    if (localUser.auxStreamID.isNotEmpty) {
      stopPublishingStream(streamType: ZegoStreamType.screenSharing);
      localUser.destroyTextureRenderer(isMainStream: false);
    }
  }

  void clear() {
    ZegoLoggerService.logInfo(
      'clear',
      tag: 'uikit',
      subTag: 'core data',
    );

    clearStream();

    isAllPlayStreamAudioVideoMuted = false;

    remoteUsersList.clear();
    streamDic.clear();
    room.clear();
  }

  ZegoUIKitCoreUser login(String id, String name) {
    ZegoLoggerService.logInfo(
      'login, id:"$id", name:$name',
      tag: 'uikit',
      subTag: 'core data',
    );
    if (id.isEmpty || name.isEmpty) {
      ZegoLoggerService.logError(
        'login params is not valid',
        tag: 'uikit',
        subTag: 'core data',
      );
    }

    localUser
      ..id = id
      ..name = name;

    userJoinStreamCtrl.add([localUser]);
    final allUserList = [localUser, ...remoteUsersList];
    userListStreamCtrl.add(allUserList);

    return localUser;
  }

  void logout() {
    ZegoLoggerService.logInfo(
      'logout',
      tag: 'uikit',
      subTag: 'core data',
    );

    localUser
      ..id = ''
      ..name = '';

    userLeaveStreamCtrl.add([localUser]);
    userListStreamCtrl.add(remoteUsersList);
  }

  void setRoom(
    String roomID, {
    bool markAsLargeRoom = false,
  }) {
    ZegoLoggerService.logInfo(
      'set room:"$roomID", markAsLargeRoom:$markAsLargeRoom}',
      tag: 'uikit',
      subTag: 'core data',
    );

    if (roomID.isEmpty) {
      ZegoLoggerService.logError(
        'room id is empty',
        tag: 'uikit',
        subTag: 'core data',
      );
    }

    room
      ..id = roomID
      ..markAsLargeRoom = markAsLargeRoom;
  }

  Future<void> startPreview() async {
    ZegoLoggerService.logInfo(
      'start preview',
      tag: 'uikit',
      subTag: 'core data',
    );

    createLocalUserVideoView(onViewCreated: () async {
      ZegoLoggerService.logInfo(
        'start preview, on view created',
        tag: 'uikit',
        subTag: 'core data',
      );

      assert(localUser.viewID != -1);

      final previewCanvas = ZegoCanvas(
        localUser.viewID,
        viewMode: videoConfig.useVideoViewAspectFill
            ? ZegoViewMode.AspectFill
            : ZegoViewMode.AspectFit,
      );

      ZegoExpressEngine.instance
        ..enableCamera(localUser.camera.value)
        ..startPreview(canvas: previewCanvas);
    });
  }

  Future<void> stopPreview() async {
    ZegoLoggerService.logInfo(
      'stop preview',
      tag: 'uikit',
      subTag: 'core data',
    );

    localUser.destroyTextureRenderer();

    ZegoExpressEngine.instance.stopPreview();
  }

  Future<void> startPublishingStream({
    required ZegoStreamType streamType,
  }) async {
    final targetStreamID = ZegoStreamType.main == streamType
        ? localUser.streamID
        : localUser.auxStreamID;
    if (targetStreamID.isNotEmpty) {
      ZegoLoggerService.logWarn(
        'startPublishingStream local user stream id($targetStreamID) is not empty',
        tag: 'uikit',
        subTag: 'core data',
      );
      return;
    }

    if (streamType == ZegoStreamType.main) {
      localUser.streamID = generateStreamID(localUser.id, room.id, streamType);
      streamDic[localUser.streamID] = localUser.id;
    } else {
      localUser.auxStreamID =
          generateStreamID(localUser.id, room.id, streamType);
      streamDic[localUser.auxStreamID] = localUser.id;
    }

    ZegoLoggerService.logInfo(
      'stream dict add ${localUser.streamID} for ${localUser.id}, $streamDic',
      tag: 'uikit',
      subTag: 'core data',
    );

    ZegoLoggerService.logInfo(
      'startPublishingStream ${localUser.streamID}',
      tag: 'uikit',
      subTag: 'core data',
    );

    createLocalUserVideoView(onViewCreated: () async {
      assert(localUser.viewID != -1);
      final previewCanvas = ZegoCanvas(
        localUser.viewID,
        viewMode: videoConfig.useVideoViewAspectFill
            ? ZegoViewMode.AspectFill
            : ZegoViewMode.AspectFit,
      );

      if (ZegoStreamType.main == streamType) {
        /// only main stream type
        ZegoExpressEngine.instance
          ..enableCamera(localUser.camera.value)
          ..muteMicrophone(!localUser.microphone.value)
          ..startPreview(canvas: previewCanvas);
      } else {
        ZegoExpressEngine.instance.setVideoConfig(
          ZegoVideoConfig.preset(ZegoVideoConfigPreset.Preset540P),
          channel: streamType.channel,
        );
      }

      ZegoExpressEngine.instance
          .startPublishingStream(
        ZegoStreamType.main == streamType
            ? localUser.streamID
            : localUser.auxStreamID,
        channel: streamType.channel,
      )
          .then((value) {
        audioVideoListStreamCtrl.add(getAudioVideoList());

        if (ZegoStreamType.screenSharing == streamType) {
          screenSharingListStreamCtrl
              .add(getAudioVideoList(streamType: ZegoStreamType.screenSharing));
        }
      });
    });
  }

  Future<void> stopPublishingStream({
    required ZegoStreamType streamType,
  }) async {
    final targetStreamID = ZegoStreamType.main == streamType
        ? localUser.streamID
        : localUser.auxStreamID;

    ZegoLoggerService.logInfo(
      'stopPublishingStream $targetStreamID}',
      tag: 'uikit',
      subTag: 'core data',
    );

    assert(targetStreamID.isNotEmpty);

    streamDic.remove(targetStreamID);
    ZegoLoggerService.logInfo(
      'stream dict remove $targetStreamID, $streamDic',
      tag: 'uikit',
      subTag: 'core data',
    );

    if (ZegoStreamType.main == streamType) {
      localUser.streamID = '';
    } else {
      localUser.auxStreamID = '';
    }

    localUser.destroyTextureRenderer(
      isMainStream: ZegoStreamType.main == streamType,
    );

    if (ZegoStreamType.main == streamType) {
      /// only main stream type
      await ZegoExpressEngine.instance.stopPreview();
    }

    await ZegoExpressEngine.instance
        .stopPublishingStream(
      channel: streamType.channel,
    )
        .then((value) {
      audioVideoListStreamCtrl.add(getAudioVideoList());
      screenSharingListStreamCtrl
          .add(getAudioVideoList(streamType: ZegoStreamType.screenSharing));
    });
  }

  Future<void> startPublishOrNot() async {
    if (room.id.isEmpty) {
      ZegoLoggerService.logError(
        'startPublishOrNot room id is empty',
        tag: 'uikit',
        subTag: 'core data',
      );
      return;
    }

    if (localUser.camera.value ||
        localUser.microphone.value ||
        localUser.microphoneMuteMode) {
      startPublishingStream(
        streamType: ZegoStreamType.main,
      );
    } else {
      if (localUser.streamID.isNotEmpty) {
        stopPublishingStream(
          streamType: ZegoStreamType.main,
        );
      }
    }
  }

  void createLocalUserVideoView({required VoidCallback onViewCreated}) {
    if (-1 == localUser.viewID) {
      ZegoExpressEngine.instance.createCanvasView((viewID) {
        localUser.viewID = viewID;
        onViewCreated();

        audioVideoListStreamCtrl.add(getAudioVideoList());
      }).then((widget) {
        localUser.view.value = widget;
      });
    } else {
      //  user view had created
      onViewCreated();
    }
  }

  ZegoUIKitCoreUser removeUser(String uid) {
    final targetIndex = remoteUsersList.indexWhere((user) => uid == user.id);
    if (-1 == targetIndex) {
      return ZegoUIKitCoreUser.empty();
    }

    final targetUser = remoteUsersList.removeAt(targetIndex);
    if (targetUser.streamID.isNotEmpty) {
      stopPlayingStream(targetUser.streamID);
    }
    if (targetUser.auxStreamID.isNotEmpty) {
      stopPlayingStream(targetUser.auxStreamID);
    }
    return targetUser;
  }

  Future<void> muteUserAudioVideo(String userID, bool mute) async {
    ZegoLoggerService.logInfo(
      'muteUserAudioVideo, userID: $userID, mute: $mute',
      tag: 'uikit',
      subTag: 'core data',
    );

    final targetUserIndex =
        remoteUsersList.indexWhere((user) => userID == user.id);
    if (-1 == targetUserIndex) {
      ZegoLoggerService.logError(
        "muteUserAudioVideo, can't find this user, userID: $userID",
        tag: 'uikit',
        subTag: 'core data',
      );
      return;
    }

    final streamID = remoteUsersList[targetUserIndex].streamID;
    if (streamID.isEmpty) {
      ZegoLoggerService.logError(
        "muteUserAudioVideo, can't find this user's stream, userID: $userID",
        tag: 'uikit',
        subTag: 'core data',
      );
      return;
    }
    ZegoExpressEngine.instance.mutePlayStreamAudio(streamID, mute);
    ZegoExpressEngine.instance.mutePlayStreamVideo(streamID, mute);
  }

  Future<void> muteAllPlayStreamAudioVideo(bool isMuted) async {
    ZegoLoggerService.logInfo(
      'mute all play stream audio video: $isMuted',
      tag: 'uikit',
      subTag: 'core data',
    );

    isAllPlayStreamAudioVideoMuted = isMuted;

    ZegoExpressEngine.instance
        .muteAllPlayStreamVideo(isAllPlayStreamAudioVideoMuted);
    ZegoExpressEngine.instance
        .muteAllPlayStreamAudio(isAllPlayStreamAudioVideoMuted);

    streamDic.forEach((streamID, userID) {
      if (isMuted) {
        // stop playing stream
        ZegoExpressEngine.instance.stopPlayingStream(streamID);
      } else {
        if (localUser.id != userID) {
          startPlayingStream(streamID, userID);
        }
      }
    });
  }

  /// will change data variables
  Future<void> startPlayingStream(String streamID, String streamUserID) async {
    ZegoLoggerService.logInfo(
      'start play stream id: $streamID, user id:$streamUserID',
      tag: 'uikit',
      subTag: 'core data',
    );

    final isMainStream = streamID.endsWith(ZegoStreamType.main.text);

    final targetUserIndex =
        remoteUsersList.indexWhere((user) => streamUserID == user.id);
    assert(-1 != targetUserIndex);
    isMainStream
        ? remoteUsersList[targetUserIndex].streamID = streamID
        : remoteUsersList[targetUserIndex].auxStreamID = streamID;

    ZegoExpressEngine.instance.createCanvasView((viewID) {
      isMainStream
          ? remoteUsersList[targetUserIndex].viewID = viewID
          : remoteUsersList[targetUserIndex].auxViewID = viewID;

      final canvas = ZegoCanvas(
        viewID,
        viewMode: isMainStream
            ? (videoConfig.useVideoViewAspectFill
                ? ZegoViewMode.AspectFill
                : ZegoViewMode.AspectFit)

            /// screen share default AspectFit
            : ZegoViewMode.AspectFit,
      );
      final playConfig = ZegoPlayerConfig(sdkPlayResourceMode);
      ZegoExpressEngine.instance
          .startPlayingStream(
        streamID,
        canvas: canvas,
        config: playConfig,
      )
          .then((value) {
        if (streamID.endsWith(ZegoStreamType.main.text)) {
          audioVideoListStreamCtrl.add(getAudioVideoList());
        } else if (streamID.endsWith(ZegoStreamType.screenSharing.text)) {
          screenSharingListStreamCtrl
              .add(getAudioVideoList(streamType: ZegoStreamType.screenSharing));
        }
      });
    }).then((widget) {
      isMainStream
          ? remoteUsersList[targetUserIndex].view.value = widget
          : remoteUsersList[targetUserIndex].auxView.value = widget;
    });
  }

  /// will change data variables
  void stopPlayingStream(String streamID) {
    ZegoLoggerService.logInfo(
      'stop play stream id: $streamID',
      tag: 'uikit',
      subTag: 'core data',
    );
    assert(streamID.isNotEmpty);

    // stop playing stream
    ZegoExpressEngine.instance.stopPlayingStream(streamID);

    final targetUserID =
        streamDic.containsKey(streamID) ? streamDic[streamID] : '';
    ZegoLoggerService.logInfo(
      'stopped play stream id $streamID, user id  is: $targetUserID',
      tag: 'uikit',
      subTag: 'core data',
    );
    final targetUserIndex =
        remoteUsersList.indexWhere((user) => targetUserID == user.id);
    if (-1 != targetUserIndex) {
      final targetUser = remoteUsersList[targetUserIndex];

      if (streamID.endsWith(ZegoStreamType.main.text)) {
        targetUser
          ..streamID = ''
          ..destroyTextureRenderer();

        targetUser.camera.value = false;
        targetUser.microphone.value = false;
        targetUser.soundLevel.add(0);
      } else if (streamID.endsWith(ZegoStreamType.screenSharing.text)) {
        targetUser
          ..auxStreamID = ''
          ..destroyTextureRenderer(isMainStream: false);
      }
    }

    // clear streamID
    streamDic.remove(streamID);
    ZegoLoggerService.logInfo(
      'stream dict remove $streamID, $streamDic',
      tag: 'uikit',
      subTag: 'core data',
    );
  }

  //start screen share
  Future<void> startSharingScreen() async {
    screenCaptureSource =
        await ZegoExpressEngine.instance.createScreenCaptureSource();

    await ZegoExpressEngine.instance
        .setVideoSource(
      ZegoVideoSourceType.ScreenCapture,
      instanceID: screenCaptureSource?.getIndex(),
      channel: ZegoPublishChannel.Aux,
    )
        .then((value) {
      isScreenSharing.value = true;
      startPublishingStream(streamType: ZegoStreamType.screenSharing);
      final config = ZegoScreenCaptureConfig(true, true, 100, 100);
      screenCaptureSource?.startCapture(config: config);
    });
    if (isFirstScreenSharing && Platform.isAndroid) {
      isFirstScreenSharing = false;
      await stopSharingScreen();
      startSharingScreen();
    }
  }

  //stop screen share
  Future<void> stopSharingScreen() async {
    isScreenSharing.value = false;

    await ZegoExpressEngine.instance.setVideoSource(ZegoVideoSourceType.None,
        channel: ZegoPublishChannel.Aux);

    await stopPublishingStream(streamType: ZegoStreamType.screenSharing);

    screenCaptureSource?.stopCapture();

    await ZegoExpressEngine.instance
        .destroyScreenCaptureSource(screenCaptureSource!);
  }

  Future<void> onRoomStreamUpdate(String roomID, ZegoUpdateType updateType,
      List<ZegoStream> streamList, Map<String, dynamic> extendedData) async {
    ZegoLoggerService.logInfo(
      'onRoomStreamUpdate, roomID:$roomID, update type:$updateType'
      ", stream list:${streamList.map((e) => "stream id:${e.streamID}, extra info${e.extraInfo}, user id:${e.user.userID}")},"
      ' extended data:$extendedData',
      tag: 'uikit',
      subTag: 'core data',
    );

    if (updateType == ZegoUpdateType.Add) {
      for (final stream in streamList) {
        streamDic[stream.streamID] = stream.user.userID;
        ZegoLoggerService.logInfo(
          'stream dict add ${stream.streamID} for ${stream.user.userID}, $streamDic',
          tag: 'uikit',
          subTag: 'core data',
        );

        if (-1 ==
            remoteUsersList
                .indexWhere((user) => stream.user.userID == user.id)) {
          /// user is not exist before stream added
          ZegoLoggerService.logInfo(
            "stream's user ${stream.user.userID}  ${stream.user.userName} is not exist, create",
            tag: 'uikit',
            subTag: 'core data',
          );

          final targetUser = ZegoUIKitCoreUser.fromZego(stream.user);
          stream.streamID.endsWith(ZegoStreamType.main.text)
              ? targetUser.streamID = stream.streamID
              : targetUser.auxStreamID = stream.streamID;
          remoteUsersList.add(targetUser);
        }

        if (isAllPlayStreamAudioVideoMuted) {
          ZegoLoggerService.logInfo(
            'audio video is not play enabled, user id:${stream.user.userID}, stream id:${stream.streamID}',
            tag: 'uikit',
            subTag: 'core data',
          );
        } else {
          await startPlayingStream(stream.streamID, stream.user.userID);
        }
      }

      onRoomStreamExtraInfoUpdate(roomID, streamList);
    } else {
      for (final stream in streamList) {
        stopPlayingStream(stream.streamID);
      }
    }

    if (-1 !=
        streamList.map((e) => e.streamID).toList().indexWhere(
            (element) => element.endsWith(ZegoStreamType.main.text))) {
      audioVideoListStreamCtrl.add(getAudioVideoList());
    }
    if (-1 !=
        streamList.map((e) => e.streamID).toList().indexWhere(
            (element) => element.endsWith(ZegoStreamType.screenSharing.text))) {
      screenSharingListStreamCtrl
          .add(getAudioVideoList(streamType: ZegoStreamType.screenSharing));
    }
  }

  List<ZegoUIKitCoreUser> getAudioVideoList({
    ZegoStreamType streamType = ZegoStreamType.main,
  }) {
    return streamDic.entries
        .where((value) => value.key.endsWith(streamType.text))
        .map((entry) {
      final targetUserID = entry.value;
      if (targetUserID == localUser.id) {
        return localUser;
      }
      return remoteUsersList.firstWhere((user) => targetUserID == user.id,
          orElse: ZegoUIKitCoreUser.empty);
    }).where((user) {
      if (user.id.isEmpty) {
        return false;
      }

      if (streamType == ZegoStreamType.main) {
        return user.camera.value || user.microphone.value;
      }

      return true;
    }).toList();
  }

  void onRoomUserUpdate(
    String roomID,
    ZegoUpdateType updateType,
    List<ZegoUser> userList,
  ) {
    ZegoLoggerService.logInfo(
      'onRoomUserUpdate, room id:"$roomID", update type:$updateType'
      "user list:${userList.map((user) => '"${user.userID}":${user.userName}, ')}",
      tag: 'uikit',
      subTag: 'core data',
    );

    if (updateType == ZegoUpdateType.Add) {
      for (final _user in userList) {
        final targetUserIndex =
            remoteUsersList.indexWhere((user) => _user.userID == user.id);
        if (-1 != targetUserIndex) {
          continue;
        }

        remoteUsersList.add(ZegoUIKitCoreUser.fromZego(_user));
      }

      if (remoteUsersList.length >= 499) {
        /// turn to be a large room after more than 500 people, even if less than 500 people behind
        ZegoLoggerService.logInfo(
          'users is more than 500, turn to be a large room',
          tag: 'uikit',
          subTag: 'core data',
        );
        room.isLargeRoom = true;
      }

      userJoinStreamCtrl.add(userList.map(ZegoUIKitCoreUser.fromZego).toList());
    } else {
      for (final user in userList) {
        removeUser(user.userID);
      }

      userLeaveStreamCtrl
          .add(userList.map(ZegoUIKitCoreUser.fromZego).toList());
    }

    final allUserList = [localUser, ...remoteUsersList];
    userListStreamCtrl.add(allUserList);
  }

  void onPublisherStateUpdate(
    String streamID,
    ZegoPublisherState state,
    int errorCode,
    Map<String, dynamic> extendedData,
  ) {
    ZegoLoggerService.logInfo(
      'onPublisherStateUpdate, stream id:$streamID, state:$state, errorCode:$errorCode, extendedData:$extendedData',
      tag: 'uikit',
      subTag: 'core data',
    );
  }

  void onPlayerStateUpdate(
    String streamID,
    ZegoPlayerState state,
    int errorCode,
    Map<String, dynamic> extendedData,
  ) {
    ZegoLoggerService.logInfo(
      'onPlayerStateUpdate, stream id:$streamID, state:$state, errorCode:$errorCode, extendedData:$extendedData',
      tag: 'uikit',
      subTag: 'core data',
    );
  }

  void onRemoteCameraStateUpdate(String streamID, ZegoRemoteDeviceState state) {
    if (!streamDic.containsKey(streamID)) {
      ZegoLoggerService.logInfo(
        'onRemoteCameraStateUpdate, stream $streamID is not exist',
        tag: 'uikit',
        subTag: 'core data',
      );
      return;
    }

    final targetUserIndex =
        remoteUsersList.indexWhere((user) => streamDic[streamID]! == user.id);
    if (-1 == targetUserIndex) {
      ZegoLoggerService.logInfo(
        'onRemoteCameraStateUpdate, stream user $streamID is not exist',
        tag: 'uikit',
        subTag: 'core data',
      );
      return;
    }

    final targetUser = remoteUsersList[targetUserIndex];
    ZegoLoggerService.logInfo(
      'onRemoteCameraStateUpdate, stream id:$streamID, user:$targetUser, state:$state',
      tag: 'uikit',
      subTag: 'core data',
    );
    final oldValue = targetUser.camera.value;
    switch (state) {
      case ZegoRemoteDeviceState.Open:
        targetUser.camera.value = true;
        break;
      case ZegoRemoteDeviceState.NoAuthorization:
        targetUser.camera.value = true;
        break;
      default:
        targetUser.camera.value = false;
    }

    if (oldValue != targetUser.camera.value) {
      /// notify outside to update audio video list
      if (streamID.endsWith(ZegoStreamType.main.text)) {
        audioVideoListStreamCtrl.add(getAudioVideoList());
      } else if (streamID.endsWith(ZegoStreamType.screenSharing.text)) {
        screenSharingListStreamCtrl
            .add(getAudioVideoList(streamType: ZegoStreamType.screenSharing));
      }
    }
  }

  void onRemoteMicStateUpdate(String streamID, ZegoRemoteDeviceState state) {
    if (!streamDic.containsKey(streamID)) {
      ZegoLoggerService.logInfo(
        'onRemoteMicStateUpdate, stream $streamID is not exist',
        tag: 'uikit',
        subTag: 'core data',
      );
      return;
    }

    final targetUserIndex =
        remoteUsersList.indexWhere((user) => streamDic[streamID]! == user.id);
    if (-1 == targetUserIndex) {
      ZegoLoggerService.logInfo(
        'onRemoteMicStateUpdate, stream user $streamID is not exist',
        tag: 'uikit',
        subTag: 'core data',
      );
      return;
    }

    final targetUser = remoteUsersList[targetUserIndex];
    ZegoLoggerService.logInfo(
      'onRemoteMicStateUpdate, stream id:$streamID, user:$targetUser, state:$state',
      tag: 'uikit',
      subTag: 'core data',
    );
    final oldValue = targetUser.microphone.value;
    switch (state) {
      case ZegoRemoteDeviceState.Open:
        targetUser.microphone.value = true;
        break;
      case ZegoRemoteDeviceState.NoAuthorization:
        targetUser.microphone.value = true;
        break;
      default:
        targetUser.microphone.value = false;
        targetUser.soundLevel.add(0);
    }

    if (oldValue != targetUser.microphone.value) {
      /// notify outside to update audio video list
      if (streamID.endsWith(ZegoStreamType.main.text)) {
        audioVideoListStreamCtrl.add(getAudioVideoList());
      } else if (streamID.endsWith(ZegoStreamType.screenSharing.text)) {
        screenSharingListStreamCtrl
            .add(getAudioVideoList(streamType: ZegoStreamType.screenSharing));
      }
    }
  }

  void onRemoteSoundLevelUpdate(Map<String, double> soundLevels) {
    soundLevels.forEach((streamID, soundLevel) {
      if (!streamDic.containsKey(streamID)) {
        if (mixerStreamDic.containsKey(streamID)) {
          return;
        }
        ZegoLoggerService.logInfo(
          'stream dic does not contain $streamID',
          tag: 'uikit',
          subTag: 'core data',
        );
        return;
      }

      final targetUserID = streamDic[streamID]!;
      final targetUserIndex =
          remoteUsersList.indexWhere((user) => targetUserID == user.id);
      if (-1 == targetUserIndex) {
        ZegoLoggerService.logInfo(
          'remote user does not contain $targetUserID',
          tag: 'uikit',
          subTag: 'core data',
        );
        return;
      }

      remoteUsersList[targetUserIndex].soundLevel.add(soundLevel);
    });
  }

  void onCapturedSoundLevelUpdate(double level) {
    localUser.soundLevel.add(level);
  }

  void onAudioRouteChange(ZegoAudioRoute audioRoute) {
    localUser.audioRoute.value = audioRoute;
  }

  void onPlayerVideoSizeChanged(String streamID, int width, int height) {
    if (!streamDic.containsKey(streamID)) {
      ZegoLoggerService.logInfo(
        'onPlayerVideoSizeChanged, stream $streamID is not exist',
        tag: 'uikit',
        subTag: 'core data',
      );
      return;
    }
    final targetUserIndex =
        remoteUsersList.indexWhere((user) => streamDic[streamID]! == user.id);
    if (-1 == targetUserIndex) {
      ZegoLoggerService.logInfo(
        'onPlayerVideoSizeChanged, stream user $streamID is not exist',
        tag: 'uikit',
        subTag: 'core data',
      );
      return;
    }

    final targetUser = remoteUsersList[targetUserIndex];
    ZegoLoggerService.logInfo(
      'onPlayerVideoSizeChanged streamID: $streamID width: $width height: $height',
      tag: 'uikit',
      subTag: 'core data',
    );
    final size = Size(width.toDouble(), height.toDouble());

    if (streamID.endsWith(ZegoStreamType.main.text)) {
      if (targetUser.viewSize.value != size) {
        targetUser.viewSize.value = size;
      }
    } else if (streamID.endsWith(ZegoStreamType.screenSharing.text)) {
      if (targetUser.auxViewSize.value != size) {
        targetUser.auxViewSize.value = size;
      }
    }
  }

  void onRoomStateChanged(String roomID, ZegoRoomStateChangedReason reason,
      int errorCode, Map<String, dynamic> extendedData) {
    ZegoLoggerService.logInfo(
      'onRoomStateChanged roomID: $roomID, reason: $reason, errorCode: $errorCode, extendedData: $extendedData',
      tag: 'uikit',
      subTag: 'core data',
    );

    room.state.value = ZegoUIKitRoomState(reason, errorCode, extendedData);

    if (reason == ZegoRoomStateChangedReason.KickOut) {
      ZegoLoggerService.logInfo(
        'local user had been kick out by room state changed',
        tag: 'uikit',
        subTag: 'core data',
      );

      meRemovedFromRoomStreamCtrl.add('');
    }
  }

  void onRoomExtraInfoUpdate(
      String roomID, List<ZegoRoomExtraInfo> roomExtraInfoList) {
    final roomExtraInfoString = roomExtraInfoList.map((info) =>
        'key:${info.key}, value:${info.value}'
        ' update user:${info.updateUser.userID},${info.updateUser.userName}, update time:${info.updateTime}');
    ZegoLoggerService.logInfo(
      'onRoomExtraInfoUpdate roomID: $roomID,roomExtraInfoList: $roomExtraInfoString',
      tag: 'uikit',
      subTag: 'core data',
    );

    for (final extraInfo in roomExtraInfoList) {
      if (extraInfo.key == 'extra_info') {
        final properties = jsonDecode(extraInfo.value) as Map<String, dynamic>;

        ZegoLoggerService.logInfo(
          'update room properties: $properties',
          tag: 'uikit',
          subTag: 'core data',
        );

        final updateProperties = <String, RoomProperty>{};

        properties.forEach((key, v) {
          final value = v as String;

          if (room.properties.containsKey(key) &&
              room.properties[key]!.value == value) {
            ZegoLoggerService.logInfo(
              'room property not need update, key:$key, value:$value',
              tag: 'uikit',
              subTag: 'core data',
            );
            return;
          }

          ZegoLoggerService.logInfo(
            'room property update, key:$key, value:$value',
            tag: 'uikit',
            subTag: 'core data',
          );
          if (room.properties.containsKey(key)) {
            final property = room.properties[key]!;
            if (extraInfo.updateTime > property.updateTime) {
              room.properties[key]!.oldValue = room.properties[key]!.value;
              room.properties[key]!.value = value;
              room.properties[key]!.updateTime = extraInfo.updateTime;
              room.properties[key]!.updateUserID = extraInfo.updateUser.userID;
            }
          } else {
            room.properties[key] = RoomProperty(
              key,
              value,
              extraInfo.updateTime,
              extraInfo.updateUser.userID,
            );
          }
          updateProperties[key] = room.properties[key]!;
          room.propertyUpdateStream.add(room.properties[key]!);
        });

        if (updateProperties.isNotEmpty) {
          room.propertiesUpdatedStream.add(updateProperties);
        }
      }
    }
  }

  void onIMRecvCustomCommand(String roomID, ZegoUser fromUser, String command) {
    ZegoLoggerService.logInfo(
      'onIMRecvCustomCommand roomID: $roomID, reason: ${fromUser.userID} ${fromUser.userName}, command:$command',
      tag: 'uikit',
      subTag: 'core data',
    );

    customCommandReceivedStreamCtrl.add(ZegoInRoomCommandReceivedData(
      fromUser: ZegoUIKitUser.fromZego(fromUser),
      command: command,
    ));
  }

  void onNetworkModeChanged(ZegoNetworkMode mode) {
    networkModeStreamCtrl.add(mode);
  }

  void startPlayAnotherRoomAudioVideo(
      String roomID, String userID, String userName) {
    ZegoLoggerService.logError(
      'startPlayAnotherRoomAudioVideo, roomID:$roomID, userID:$userID, userName:$userName',
      tag: 'uikit',
      subTag: 'core',
    );

    var targetUserIndex =
        remoteUsersList.indexWhere((user) => userID == user.id);
    if (-1 == targetUserIndex) {
      remoteUsersList
          .add(ZegoUIKitCoreUser(userID, userName)..isAnotherRoomUser = true);
    }
    targetUserIndex = remoteUsersList.indexWhere((user) => userID == user.id);

    final streamID = generateStreamID(userID, roomID, ZegoStreamType.main);
    streamDic[streamID] = userID;
    remoteUsersList[targetUserIndex].streamID = streamID;

    ZegoExpressEngine.instance.createCanvasView((viewID) {
      remoteUsersList[targetUserIndex].viewID = viewID;
      final canvas = ZegoCanvas(viewID, viewMode: ZegoViewMode.AspectFill);
      ZegoExpressEngine.instance.startPlayingStream(streamID, canvas: canvas);
    }).then((widget) {
      assert(widget != null);
      remoteUsersList[targetUserIndex].view.value = widget;
    });
  }

  void stopPlayAnotherRoomAudioVideo(String userID) {
    ZegoLoggerService.logInfo(
      'stopPlayAnotherRoomAudioVideo, userID:$userID',
      tag: 'uikit',
      subTag: 'core data',
    );

    final targetUserIndex =
        remoteUsersList.indexWhere((user) => userID == user.id);
    if (-1 != targetUserIndex) {
      final targetUser = remoteUsersList[targetUserIndex];
      final streamID = remoteUsersList[targetUserIndex].streamID;
      ZegoExpressEngine.instance.stopPlayingStream(streamID);

      targetUser
        ..streamID = ''
        ..destroyTextureRenderer();

      targetUser.camera.value = false;
      targetUser.microphone.value = false;
      targetUser.soundLevel.add(0);
      streamDic.remove(streamID);
      remoteUsersList.removeWhere((user) => userID == user.id);
      ZegoLoggerService.logInfo(
        'stopPlayAnotherRoomAudioVideo, userID:$userID, streamID:$streamID',
        tag: 'uikit',
        subTag: 'core data',
      );
    } else {
      ZegoLoggerService.logWarn(
        "stopPlayAnotherRoomAudioVideo, can't find this user, userID:$userID",
        tag: 'uikit',
        subTag: 'core data',
      );
    }
  }

  void muteAudio(String userID, bool mute) {
    ZegoLoggerService.logInfo(
      'muteAudio, userID:$userID, mute:$mute',
      tag: 'uikit',
      subTag: 'core data',
    );
    final targetUserIndex =
        remoteUsersList.indexWhere((user) => userID == user.id);
    if (-1 != targetUserIndex) {
      final streamID = remoteUsersList[targetUserIndex].streamID;
      ZegoExpressEngine.instance.mutePlayStreamAudio(streamID, mute);
      ZegoLoggerService.logInfo(
        'muteAudio, userID:$userID, streamID:$streamID',
        tag: 'uikit',
        subTag: 'core data',
      );
    } else {
      ZegoLoggerService.logWarn(
        "muteAudio, can't find this user, userID:$userID",
        tag: 'uikit',
        subTag: 'core data',
      );
    }
  }

  Future<void> startPlayMixAudioVideo(
      String mixerID, List<String> users) async {
    ZegoLoggerService.logWarn(
      'startPlayMixAudioVideo, mixerID:$mixerID, users:$users',
      tag: 'uikit',
      subTag: 'core data',
    );
    mixerStreamDic[mixerID] = ZegoUIKitCoreMixerStream(
      mixerID,
      users.map((e) => ZegoUIKitCoreUser(e, '')).toList(),
    );
    ZegoExpressEngine.instance.createCanvasView((viewID) {
      mixerStreamDic[mixerID]!.viewID = viewID;
      final canvas = ZegoCanvas(viewID, viewMode: ZegoViewMode.AspectFill);
      ZegoExpressEngine.instance.startPlayingStream(mixerID, canvas: canvas);
      Future.delayed(const Duration(seconds: 3), () {
        mixerStreamDic[mixerID]?.loaded.value = true;
      });
    }).then((widget) {
      assert(widget != null);
      mixerStreamDic[mixerID]!.view.value = widget;
    });
  }

  Future<void> stopPlayMixAudioVideo(String mixerID) async {
    ZegoLoggerService.logInfo(
      'stopPlayMixAudioVideo, mixerID:$mixerID',
      tag: 'uikit',
      subTag: 'core data',
    );
    ZegoExpressEngine.instance.stopPlayingStream(mixerID);
    mixerStreamDic[mixerID]!.destroyTextureRenderer();
    mixerStreamDic.remove(mixerID);
  }

  void onPlayerRecvVideoFirstFrame(String mixerID) {
    mixerStreamDic[mixerID]?.loaded.value = true;
  }

  void onPlayerRecvAudioFirstFrame(String mixerID) {
    mixerStreamDic[mixerID]?.loaded.value = true;
  }

  void sendSEI(String typeIdentifier, Map<String, dynamic> sei) {
    final dataJson = jsonEncode({
      streamSEIKeyUserID: localUser.id,
      streamSEIKeyTypeIdentifier: typeIdentifier,
      streamSEIKeySEI: sei,
    });
    final dataBytes = Uint8List.fromList(utf8.encode(dataJson));
    ZegoExpressEngine.instance.sendSEI(dataBytes, dataBytes.length);
  }

  void onPlayerRecvSEI(String streamID, Uint8List data) {
    final dataJson = utf8.decode(data.toList());
    try {
      final dataMap = jsonDecode(dataJson) as Map<String, dynamic>;
      final typeIdentifier = dataMap[streamSEIKeyTypeIdentifier] as String;
      final sei = dataMap[streamSEIKeySEI] as Map<String, dynamic>;
      final uid = dataMap[streamSEIKeyUserID] as String;
      if (typeIdentifier == ZegoUIKitInnerSEIType.zegoMixerDeviceState.name &&
          streamID.endsWith(ZegoStreamType.mix.text)) {
        _updateMixerDeviceStateBySEI(streamID, uid, sei);
      }
      receiveSEIStreamCtrl
          .add(ZegoUIKitReceiveSEIEvent(typeIdentifier, uid, sei));
    } catch (e) {
      ZegoLoggerService.logWarn(
        'onPlayerRecvSEI, decode sei failed, sei: $dataJson',
        tag: 'uikit',
        subTag: 'core data',
      );
    }
  }

  void _updateMixerDeviceStateBySEI(
      String streamID, String userID, Map<String, dynamic> sei) {
    assert(mixerStreamDic.containsKey(streamID),
        'onPlayerRecvSEI, mixerStreamDic not contains streamID:$streamID');
    final userIndex =
        mixerStreamDic[streamID]!.users.indexWhere((user) => user.id == userID);
    if (userIndex == -1) {
      final user = ZegoUIKitCoreUser(userID, '');
      user.camera.value = sei[streamSEIKeyCamera]!;
      user.microphone.value = sei[streamSEIKeyMicrophone]!;
      mixerStreamDic[streamID]!.users.add(user);
    } else {
      final user = mixerStreamDic[streamID]!.users[userIndex];
      user.camera.value = sei[streamSEIKeyCamera]!;
      user.microphone.value = sei[streamSEIKeyMicrophone]!;
    }
  }

  void onRoomStreamExtraInfoUpdate(String roomID, List<ZegoStream> streamList) {
    /*
    * {
    * "isCameraOn": true,
    * "isMicrophoneOn": true,
    * "hasAudio": true,
    * "hasVideo": true,
    * }
    * */

    ZegoLoggerService.logInfo(
      "onRoomStreamExtraInfoUpdate, roomID:$roomID, stream list:${streamList.map((e) => "stream id:${e.streamID}, extra info${e.extraInfo}, user id:${e.user.userID}")}",
      tag: 'uikit',
      subTag: 'core data',
    );
    for (final stream in streamList) {
      if (stream.extraInfo.isEmpty) {
        ZegoLoggerService.logInfo(
          'onRoomStreamExtraInfoUpdate extra info is empty',
          tag: 'uikit',
          subTag: 'core data',
        );
        continue;
      }

      final extraInfos = jsonDecode(stream.extraInfo) as Map<String, dynamic>;
      if (extraInfos.containsKey(streamExtraInfoCameraKey)) {
        onRemoteCameraStateUpdate(
            stream.streamID,
            extraInfos[streamExtraInfoCameraKey]!
                ? ZegoRemoteDeviceState.Open
                : ZegoRemoteDeviceState.Mute);
      }
      if (extraInfos.containsKey(streamExtraInfoMicrophoneKey)) {
        onRemoteMicStateUpdate(
            stream.streamID,
            extraInfos[streamExtraInfoMicrophoneKey]!
                ? ZegoRemoteDeviceState.Open
                : ZegoRemoteDeviceState.Mute);
      }
    }
  }
}
